import type { ConfigConditions, ConfigStaticProperties, ConfigDynamicProperties, ConfigShorthands, CommonOptions } from "./types.js";
declare type ConditionalMap<Conditions> = {
    default: string;
    conditions: Record<keyof Conditions, string>;
};
declare type ReturnConditionalDynamic<DynamicProperties extends ConfigDynamicProperties, Conditions extends ConfigConditions> = {
    config: {
        [Property in keyof DynamicProperties]: {
            dynamic: ConditionalMap<Conditions>;
            dynamicScale: DynamicProperties[Property];
            name: Property;
            vars: ConditionalMap<Conditions>;
        };
    };
};
declare type ReturnDynamic<DynamicProperties extends ConfigDynamicProperties> = {
    config: {
        [Property in keyof DynamicProperties]: {
            dynamic: {
                default: string;
            };
            dynamicScale: DynamicProperties[Property];
            name: Property;
            vars: {
                default: string;
            };
        };
    };
};
declare type Values<Property, Result> = {
    [Value in Property extends ReadonlyArray<any> ? Property[number] : Property extends Array<any> ? Property[number] : keyof Property]: Result;
};
declare type ReturnConditionalStatic<StaticProperties extends ConfigStaticProperties, Conditions extends ConfigConditions> = {
    config: {
        [Property in keyof StaticProperties]: {
            values: Values<StaticProperties[Property], ConditionalMap<Conditions>>;
            staticScale: StaticProperties[Property];
            name: Property;
        };
    };
};
declare type ReturnStatic<StaticProperties extends ConfigStaticProperties> = {
    config: {
        [Property in keyof StaticProperties]: {
            values: Values<StaticProperties[Property], {
                default: string;
            }>;
            staticScale: StaticProperties[Property];
            name: Property;
        };
    };
};
declare type ReturnShorthands<Shorthands extends {
    [k: string]: Array<string | number | symbol>;
}> = {
    config: {
        [Shorthand in keyof Shorthands]: {
            mappings: Shorthands[Shorthand];
        };
    };
};
declare type ShorthandOptions<Properties extends object, Shorthands extends {
    [shorthandName: string]: Array<keyof Properties>;
}> = {
    shorthands?: Shorthands;
};
export declare type OptionsConditionalDynamic<DynamicProperties extends ConfigDynamicProperties, Conditions extends ConfigConditions> = {
    dynamicProperties: DynamicProperties;
    conditions: Conditions;
    defaultCondition: keyof Conditions;
};
export declare type OptionsConditionalStatic<StaticProperties extends ConfigStaticProperties, Conditions extends ConfigConditions> = {
    staticProperties: StaticProperties;
    conditions: Conditions;
    defaultCondition: keyof Conditions;
};
export declare type OptionsConditionalBoth<DynamicProperties extends ConfigDynamicProperties, StaticProperties extends ConfigStaticProperties, Conditions extends ConfigConditions> = {
    dynamicProperties: DynamicProperties;
    staticProperties: StaticProperties;
    conditions: Conditions;
    defaultCondition: keyof Conditions;
};
export declare type OptionsDynamic<DynamicProperties extends ConfigDynamicProperties> = {
    dynamicProperties: DynamicProperties;
};
export declare type OptionsStatic<StaticProperties extends ConfigStaticProperties> = {
    staticProperties: StaticProperties;
};
export declare type OptionsBoth<DynamicProperties extends ConfigDynamicProperties, StaticProperties extends ConfigStaticProperties> = {
    dynamicProperties: DynamicProperties;
    staticProperties: StaticProperties;
};
/**
 * NO SHORTHANDS + NO CONDITIONS
 */
export declare function defineProperties<DynamicProperties extends ConfigDynamicProperties>(options: CommonOptions & OptionsDynamic<DynamicProperties>): ReturnDynamic<DynamicProperties>;
export declare function defineProperties<StaticProperties extends ConfigStaticProperties>(options: CommonOptions & OptionsStatic<StaticProperties>): ReturnStatic<StaticProperties>;
export declare function defineProperties<DynamicProperties extends ConfigDynamicProperties, StaticProperties extends ConfigStaticProperties>(options: CommonOptions & OptionsBoth<DynamicProperties, StaticProperties>): ReturnStatic<StaticProperties> & ReturnDynamic<DynamicProperties>;
/**
 * SHORTHANDS + NO CONDITIONS
 */
export declare function defineProperties<DynamicProperties extends ConfigDynamicProperties, Shorthands extends {
    [k: string]: Array<keyof DynamicProperties>;
}>(options: CommonOptions & OptionsDynamic<DynamicProperties> & ShorthandOptions<DynamicProperties, Shorthands>): ReturnDynamic<DynamicProperties> & ReturnShorthands<Shorthands>;
export declare function defineProperties<StaticProperties extends ConfigStaticProperties, Shorthands extends {
    [k: string]: Array<keyof StaticProperties>;
}>(options: CommonOptions & OptionsStatic<StaticProperties> & ShorthandOptions<StaticProperties, Shorthands>): ReturnStatic<StaticProperties> & ReturnShorthands<Shorthands>;
export declare function defineProperties<DynamicProperties extends ConfigDynamicProperties, StaticProperties extends ConfigStaticProperties, Shorthands extends ConfigShorthands<DynamicProperties, StaticProperties>>(options: CommonOptions & OptionsBoth<DynamicProperties, StaticProperties> & ShorthandOptions<DynamicProperties & StaticProperties, Shorthands>): ReturnStatic<StaticProperties> & ReturnDynamic<DynamicProperties> & ReturnShorthands<Shorthands>;
/**
 * NO SHORTHANDS + CONDITIONS
 */
export declare function defineProperties<DynamicProperties extends ConfigDynamicProperties, Conditions extends ConfigConditions>(options: CommonOptions & OptionsConditionalDynamic<DynamicProperties, Conditions>): ReturnConditionalDynamic<DynamicProperties, Conditions>;
export declare function defineProperties<StaticProperties extends ConfigStaticProperties, Conditions extends ConfigConditions>(options: CommonOptions & OptionsConditionalStatic<StaticProperties, Conditions>): ReturnConditionalStatic<StaticProperties, Conditions>;
export declare function defineProperties<DynamicProperties extends ConfigDynamicProperties, StaticProperties extends ConfigStaticProperties, Conditions extends ConfigConditions>(options: CommonOptions & OptionsConditionalBoth<DynamicProperties, StaticProperties, Conditions>): ReturnConditionalStatic<StaticProperties, Conditions> & ReturnConditionalDynamic<DynamicProperties, Conditions>;
/**
 * SHORTHANDS + CONDITIONS
 */
export declare function defineProperties<DynamicProperties extends ConfigDynamicProperties, Conditions extends ConfigConditions, Shorthands extends {
    [k: string]: Array<keyof DynamicProperties>;
}>(options: CommonOptions & OptionsConditionalDynamic<DynamicProperties, Conditions> & ShorthandOptions<DynamicProperties, Shorthands>): ReturnConditionalDynamic<DynamicProperties, Conditions> & ReturnShorthands<Shorthands>;
export declare function defineProperties<StaticProperties extends ConfigStaticProperties, Conditions extends ConfigConditions, Shorthands extends {
    [k: string]: Array<keyof StaticProperties>;
}>(options: CommonOptions & OptionsConditionalStatic<StaticProperties, Conditions> & ShorthandOptions<StaticProperties, Shorthands>): ReturnConditionalStatic<StaticProperties, Conditions> & ReturnShorthands<Shorthands>;
export declare function defineProperties<DynamicProperties extends ConfigDynamicProperties, StaticProperties extends ConfigStaticProperties, Conditions extends ConfigConditions, Shorthands extends ConfigShorthands<DynamicProperties, StaticProperties>>(options: CommonOptions & OptionsConditionalBoth<DynamicProperties, StaticProperties, Conditions> & ShorthandOptions<DynamicProperties & StaticProperties, Shorthands>): ReturnConditionalStatic<StaticProperties, Conditions> & ReturnConditionalDynamic<DynamicProperties, Conditions> & ReturnShorthands<Shorthands>;
export {};
